#include <msp/fs/stat.h>
#include <msp/fs/utils.h>
#include "builder.h"
#include "installedfile.h"
#include "sharedlibrary.h"

using namespace std;
using namespace Msp;

InstalledFile::InstalledFile(Builder &b, const SourcePackage &p, FileTarget &s, const string &loc):
	FileTarget(b, p, generate_target_path(b.get_prefix(), s, loc)),
	source(s)
{
	add_dependency(source);
}

FS::Path InstalledFile::generate_target_path(const FS::Path &prefix, const FileTarget &tgt, const string &loc)
{
	if(!tgt.is_installable() && loc.empty())
		throw invalid_argument(tgt.get_name()+" is not installable");

	FS::Path mid;
	if(!loc.empty())
		mid = loc;
	else if(const Component *comp = tgt.get_component())
		mid = comp->get_install_map().get_install_location(tgt);
	else
		mid = tgt.get_install_location();

	string fn = tgt.get_install_filename();
	if(fn.empty())
		fn = FS::basename(tgt.get_path());

	return prefix/mid/fn;
}

void InstalledFile::set_symlink(const FS::Path &l)
{
	FS::Path al = FS::dirname(path)/l;
	if(al==path)
		throw invalid_argument("InstalledFile::set_symlink");
	link = FS::dirname(path)/l;
	builder.get_vfs().register_path(link, this);
}

Target *InstalledFile::get_real_target()
{
	return source.get_real_target();
}

void InstalledFile::check_rebuild()
{
	if(!mtime)
		mark_rebuild("Does not exist");
	else if(source.get_mtime()>mtime || source.get_size()!=size)
		mark_rebuild(source.get_name()+" has changed");
	else if(source.needs_rebuild())
		mark_rebuild(source.get_name()+" needs rebuilding");
	if(!needs_rebuild() && !link.empty())
	{
		if(!FS::exists(link))
			mark_rebuild("Symlink does not exist");
		else
		{
			FS::Path rel_path = FS::relative(path, FS::dirname(link));
			if(FS::readlink(link)!=rel_path)
				mark_rebuild("Symlink needs updating");
		}
	}
}

void InstalledFile::clean()
{
	if(!link.empty() && mtime)
		FS::unlink(link);
	FileTarget::clean();
}
