#include <algorithm>
#include <msp/strings/format.h>
#include "architecture.h"
#include "builder.h"
#include "tool.h"

using namespace std;
using namespace Msp;

Tool::Tool(Builder &b, const string &t):
	builder(b),
	architecture(0),
	tag(t),
	executable(0),
	prepared(false)
{ }

Tool::Tool(Builder &b, const Architecture &a, const string &t):
	builder(b),
	architecture(&a),
	tag(t),
	executable(0),
	prepared(false)
{ }

void Tool::set_command(const string &cmd, bool cross)
{
	if(cmd.empty())
		throw invalid_argument("Tool::set_command");

	if(cross && architecture->is_cross() && !FS::Path(cmd).is_absolute())
		command = format("%s-%s", architecture->get_cross_prefix(), cmd);
	else
		command = cmd;
}

bool Tool::accepts_suffix(const string &suffix, bool aux) const
{
	if(find(input_suffixes.begin(), input_suffixes.end(), suffix)!=input_suffixes.end())
		return true;
	else if(aux)
		return find(aux_suffixes.begin(), aux_suffixes.end(), suffix)!=aux_suffixes.end();
	else
		return false;
}

Target *Tool::create_target(Target &source, const string &arg)
{
	list<Target *> sources;
	sources.push_back(&source);
	return create_target(sources, arg);
}

void Tool::prepare()
{
	if(prepared)
		return;

	prepared = true;
	do_prepare();
	if(!executable && !command.empty())
	{
		executable = builder.get_vfs().find_binary(command);
		if(!executable)
			problems.push_back(format("Can't find executable %s", command));
	}
}


SubTool::SubTool(Tool &p):
	Tool(p),
	parent(p)
{ }
